package de.hft.stuttgart.citydoctor2.webservice.utils;

import java.time.LocalDateTime;

import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.core.HttpHeaders;

import de.hft.stuttgart.citydoctor2.webservice.AuthenticationException;
import de.hft.stuttgart.citydoctor2.webservice.database.TokenQueries;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.Token;
import de.hft.stuttgart.citydoctor2.webservice.model.entities.User;

public class AuthenticationUtils {
	
	public static final String AUTHENTICATION_DENIED = "Authentication denied for address {}, reason: {}";
	
	private AuthenticationUtils() {
		// only static use
	}
	
	public static User authenticateUser(HttpServletRequest request, EntityManager manager) throws AuthenticationException {
		final String authorization = request.getHeader(HttpHeaders.AUTHORIZATION);
		if (authorization != null && authorization.toLowerCase().startsWith("bearer")) {
			// Authorization: Bearer token
			String token = authorization.substring("Bearer".length()).trim();
			return validateToken(token, request.getRemoteAddr(), manager);
		}
		throw new AuthenticationException("No bearer token provided with request");
	}

	private static User validateToken(String token, String remoteAddr, EntityManager manager) throws AuthenticationException {
		Token tokenForUser = TokenQueries.getTokenForTokenString(token, remoteAddr, manager);
		if (tokenForUser == null) {
			throw new AuthenticationException("No token for user");
		}
		if (!tokenForUser.getTokenString().equals(token)) {
			throw new AuthenticationException("Token does not match");
		}
		LocalDateTime now = LocalDateTime.now();
		if (tokenForUser.getExpires().isBefore(now)) {
			throw new AuthenticationException("Token expired");
		}
		// update expiration date
		TokenQueries.updateExpirationDate(now.plusHours(3), tokenForUser, manager);
		return tokenForUser.getUser();
	}
	
	public static void removeTokenFromUser(HttpServletRequest request, EntityManager manager) throws AuthenticationException {
		final String authorization = request.getHeader(HttpHeaders.AUTHORIZATION);
		if (authorization != null && authorization.toLowerCase().startsWith("bearer")) {
			String token = authorization.substring("Bearer".length()).trim();
			Token tokenForUser = TokenQueries.getTokenForTokenString(token, request.getRemoteAddr(), manager);
			if (tokenForUser == null) {
				throw new AuthenticationException("No token for user");
			}
			manager.getTransaction().begin();
			manager.remove(tokenForUser);
			manager.getTransaction().commit();
			return;
		}
		throw new AuthenticationException("No bearer token provided with request");
	}
 
}
